@@ -2,9 +2,13 @@
 from django.db.models.query_utils import Q
 from django.conf import settings
 
-from m3.core.json import M3JSONEncoder
-from m3.ui.actions import Action, PreJsonResult, OperationResult, ActionPack, ActionController
-from m3.ui.actions.tree_packs import BaseTreeDictionaryModelActions
+from m3 import M3JSONEncoder
+from m3.actions import (Action,
+                        PreJsonResult,
+                        OperationResult,
+                        ActionPack,
+                        ActionController,)
+from m3.actions.tree_packs import BaseTreeDictionaryModelActions
 
 from models import KladrGeo, KladrStreet
 from ui import KLADRGeoEditWindow, KLADRStreetEditWindow
@@ -25,21 +29,21 @@ class Kladr_DictPack(BaseTreeDictionaryModelActions):
     verbose_name = u'Классификатор адресов России (КЛАДР)'
     tree_model = KladrGeo
     list_model = KladrStreet
-    
+
     list_columns = [('socr', u'Сокращение', 20),
                     ('name', u'Улица')]
     tree_columns = [('name', u'Геогр. пункт', 240),
                     ('socr', u'Сокращение', 90),
                     ('zipcode', u'Индекс', 60)]
-    
+
     filter_fields = ['code', 'name']
     tree_filter_fields = ['name', 'zipcode']
-    
+
     width, height = 910, 600
     tree_width = 390
-    
+
     edit_window = KLADRGeoEditWindow
-    edit_node_window = KLADRStreetEditWindow 
+    edit_node_window = KLADRStreetEditWindow
 
 class KLADRPack(ActionPack):
     url = ''
@@ -70,7 +74,7 @@ class KLADRPack(ActionPack):
     def get_street_name(self, code):
         street = KladrStreet.objects.select_related('parent').filter(code = code)
         return street[0].display_name() if street else ''
-    
+
     @classmethod
     def get_street(self, code):
         street = KladrStreet.objects.select_related('parent').filter(code = code)
@@ -81,7 +85,7 @@ class KLADRRowsAction(Action):
     Перечисление элементов КЛАДРа
     '''
     url = '/kladr_rows$'
-    
+
     def run(self, request, context):
         filter = request.REQUEST.get('filter')
         if filter: # бывают случаи, когда фильтр приходит пустой
@@ -121,58 +125,72 @@ class KLADRRowsAction(Action):
         result = {'rows': list(places), 'total': len(places)}
         return PreJsonResult(result)
 
+
 class StreetRowsAction(Action):
-    '''
+    """
     Перечисление улиц
-    '''
+    """
     url = '/street_rows$'
-    
+
+    MAX_DEPTH = 3  # макс. глубина вложенности детей в родителя
+
     def run(self, request, context):
-        filter = request.REQUEST.get('filter')
-        if filter: # бывают случаи, когда фильтр приходит пустой
+        filt = request.REQUEST.get('filter')
+        if filt:  # бывают случаи, когда фильтр приходит пустой
             place_code = request.REQUEST.get('place_code')
-            place_id = None
+            place_filter = Q()
             if place_code:
-                place = KladrGeo.objects.filter(code=place_code)
-                if place and len(place) == 1:
-                    place_id = place[0].id
-            fields = ['name','code','socr'];
-            words = filter.strip().split(' ')
-            # первым этапом найдем территории подходящие под фильтр в имени
-            condition = None
-            for word in words:
-                field_condition = None
-                for field_name in fields:
-                    field = Q(**{field_name + '__icontains': word})
-                    field_condition = field_condition | field if field_condition else field
-                condition = condition & field_condition if condition else field_condition
-            if place_id:
-                places = KladrStreet.objects.filter(condition, parent = place_id).\
-                         select_related('parent').\
-                         order_by('name')[0:50]
-            else:
-                places = KladrStreet.objects.filter(condition).\
-                         select_related('parent').\
-                         order_by('name')[0:50]
-            # если не нашли, то будем искать с учетом родителей
-            if len(places) == 0:
-                condition = None
-                for word in words:
-                    field_condition = None
-                    for field_name in fields:
-                        field = Q(**{field_name + '__icontains': word}) |\
-                                Q(**{'parent__' + field_name + '__icontains': word}) |\
-                                Q(**{'parent__parent__' + field_name + '__icontains': word}) |\
-                                Q(**{'parent__parent__parent__' + field_name + '__icontains': word})
-                        field_condition = field_condition | field if field_condition else field
-                    condition = condition & field_condition if condition else field_condition
-                if place_id:
-                    places = KladrStreet.objects.filter(condition, parent = place_id).\
-                             select_related('parent').order_by('name')[0:50]
+                try:
+                    place_id = KladrGeo.objects.get(code=place_code)
+                except (
+                    KladrGeo.DoesNotExist,
+                    KladrGeo.MultipleObjectsReturned
+                ):
+                    pass
                 else:
-                    places = KladrStreet.objects.filter(condition).\
-                             select_related('parent').\
-                             order_by('name')[0:50]
+                    for depth in range(self.MAX_DEPTH):
+                        place_filter |= Q(**{
+                            '%sparent' % ('parent__' * depth): place_id
+                        })
+
+            fields = ['name', 'code', 'socr']
+            words = filt.strip().split()
+
+            # формирователь выборки улиц
+            def query_streets(condition):
+                return KladrStreet.objects.filter(
+                    condition,
+                    place_filter,
+                ).select_related(
+                    'parent'
+                ).order_by(
+                    'name'
+                )[0:50]
+
+            # первым этапом найдем территории подходящие под фильтр в имени
+            condition = Q()
+            for word in words:
+                field_condition = Q()
+                for field_name in fields:
+                    field_condition |= Q(**{field_name + '__icontains': word})
+                condition &= field_condition
+            places = query_streets(condition)
+
+            # если не нашли, то будем искать с учетом родителей
+            if not places.exists():
+                condition = Q()
+                for word in words:
+                    field_condition = Q()
+                    for field_name in fields:
+                        for depth in range(self.MAX_DEPTH + 1):
+                            field_condition |= Q(**{
+                                '%s%s__icontains' % (
+                                    'parent__' * depth,
+                                    field_name
+                                ): word
+                            })
+                    condition &= field_condition
+                places = query_streets(condition)
         else:
             places = []
         result = {'rows': list(places), 'total': len(places)}
@@ -193,7 +211,7 @@ def GetAddr(place, street = None, house = None, flat = None, zipcode = None, cor
             return ''
     elif not isinstance(place, KladrGeo):
         raise TypeError()
-        
+
     # Получаем улицу
     if isinstance(street, (str, unicode)):
         qs = KladrStreet.objects.filter(code=street)
@@ -293,7 +311,7 @@ def GetAddr(place, street = None, house = None, flat = None, zipcode = None, cor
             addr_text = delim+delim+addr_text
     # если нет дома и корпуса и квартиры, то уберем последний разделитель
     if not (house or corps or flat):
-        addr_text = addr_text.rstrip(delim) 
+        addr_text = addr_text.rstrip(delim)
     else:
         if house:
             addr_text = addr_text+u'д. '+house
@@ -309,7 +327,7 @@ class KLADRGetAddrAction(Action):
     Расчет адреса по составляющим
     '''
     url = '/kladr_getaddr$'
-    
+
     def run(self, request, context):
         place = request.REQUEST.get('place')
         street = request.REQUEST.get('street')
