@@ -2,14 +2,15 @@
 
 from django.conf import urls
 
-from m3.ui.actions import ActionController
 from actions import kladr_controller, KLADRPack
-from m3.helpers.users import authenticated_user_required
+from m3 import authenticated_user_required
+
 
 def register_actions():
     kladr_controller.packs.append(KLADRPack)
     #kladr_controller.rebuild_patterns()
 
+
 def register_urlpatterns():
     '''
     Регистрация конфигурации урлов для приложения kladr
